-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;

local lastLooted = "";

variables.events = 
{
    ["CHAT_MSG_RAID_WARNING"] = 
    function(...)
        local msg, author = ...;
        local name, _ = strsplit("-", author);
        
        --Check if the person sending a raidwarning is the raid leader or if assist setting is enabled.
        if not (UnitIsGroupLeader(name) or llrSettings.assistMode) then
            return;
        end

        --Hide user window as new item is being rolled.
        functions.hideUser();
        
        local _, link, _, _, _, _, _, _, itemEquipLoc, _, _, itemClassID, itemSubClassID = GetItemInfo(msg);
        local isPet = itemClassID == 15 and itemSubClassID == 2;

        if not 
        (
            itemClassID == 4 or --Armor
            itemClassID == 2 or --Weapon
            isPet 				--Pet
        )
        then
            return;
        end

        local usabilityVar = 
        --Check if it is a pet.
        isPet or
        --Check if it item is a cloak.
        itemEquipLoc == "INVTYPE_CLOAK" or 
        --Check if it is a ring.
        itemEquipLoc == "INVTYPE_FINGER" or 
        --Check if it is a neck.
        itemEquipLoc == "INVTYPE_NECK" or 
        --Check if it is a trinket.
        itemEquipLoc == "INVTYPE_TRINKET" or
        --Check if item is an off-hand, and if player should use off-hands.
        (itemEquipLoc == "INVTYPE_HOLDABLE" and variables.classArray[variables.playerClassId][itemEquipLoc]) or
        --Check if item is an equippable weapon/armor piece with correct stats.
        (variables.classArray[variables.playerClassId][itemClassID][itemSubClassID] and functions.usableStats(link)) or 
        --Check if collectable mog.
        functions.usableMog(link);

        if UnitIsGroupLeader("player") or llrSettings.leadMode then
            functions.historyLengthControl();

            --Insert the new item into the rollhistory array.
            tinsert(
                variables.rollHistory, 
                1, 
                {
                    item 		= link,
                    mainList 	= {},
                    offList 	= {},
                    mogList 	= {},
                    otherList   = {},
                    distributed = false
                }
            );

            functions.leaderEvent(usabilityVar);
        elseif usabilityVar then
            functions.userEvent(link);
        end
    end,
    ["ENCOUNTER_LOOT_RECEIVED"] = 
    function(...)
        local _, _, item, _, name = ...;
			
        if name == variables.playerName then
            print("Looted:", item);
            lastLooted = item;
        end
    end,
    ["BAG_UPDATE"] = 
    function(...)
        if lastLooted == "" then
            return;
        end

        local bag = ...;

        for slot = 1, GetContainerNumSlots(bag) do
            local bagLink = GetContainerItemLink(bag, slot);

            if bagLink and bagLink == lastLooted then
                if functions.scanToolTipAndQueue(bag, slot, bagLink) then 
                    break;
                end
            end
        end

        lastLooted = "";
    end,
    ["GROUP_ROSTER_UPDATE"] = 
    function(...)
        functions.toggleLootListener();
        functions.toggleWhisperListener();
        functions.controlRollListener();
    end,
    ["CHAT_MSG_SYSTEM"] = 
    function(...)
        if not variables.rollHistory[1] or variables.rollHistory[1].distributed then
			return;
		end

		local msg = ...;
		local author, roll, rollMin, rollMax = strmatch(msg, "(.+) rolls (%d+) %((%d+)-(%d+)%)");
		
		if not author then
			return;
		end
		
		--Convert the rolls to numbers.
		roll 	= tonumber(roll);
		rollMin = tonumber(rollMin);
        rollMax = tonumber(rollMax);

		if not roll or not rollMax or rollMin ~= 1 or rollMax > variables.maxRollValue then 
			return;
		end

		local color = functions.getClassColor(author);
		local roller = {author, roll, color.r, color.g, color.b};
        local determinedArray = variables.rollHistory[1].otherList;
        local rollType = "other";

		if rollMax == llrSettings.mainRoll then
            determinedArray = variables.rollHistory[1].mainList;
            rollType = "main";
		elseif rollMax == llrSettings.offRoll then
			determinedArray = variables.rollHistory[1].offList;
            rollType = "off";
		elseif rollMax == llrSettings.mogRoll then
			determinedArray = variables.rollHistory[1].mogList;
            rollType = "transmog";
		else
			roller[2] = rollMax;
		end

		--Determine if user has already rolled, and if found then exit function.
		if #determinedArray > 0 then
			for _,v in ipairs(determinedArray) do 
				if v[1] == roller[1] then
					return;
				end
			end
        end
        
        tinsert(roller, rollType);
		
		--Clear the current frame and add the roller to the list.
		tinsert(determinedArray, roller);
    end,
    ["CHAT_MSG_WHISPER"] = 
    function(...)
        local msg, author = ...;
        local name, _ = strsplit("-", author);

        if UnitPlayerOrPetInRaid(name) then
            functions.registerItem(name, msg);
        end
    end
}