-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;

local masterLooter = "";


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for transmitting the tradeable item to the leader/player's itemList.
local function sendTradeableFunc(self)
	local input = self.editBox:GetText();

	if UnitPlayerOrPetInRaid(input) then
		local _, link = GetItemInfo(self.data);
		
		if input == variables.playerName then
			functions.registerItem(input, link);
		else
			SendChatMessage(link, "WHISPER", nil, input);
		end

		masterLooter = input;
	else
		functions.printOut("Player is not in the raid group.");
	end
end

--Function for handling manual rolls.
local function manualRoll(self)
	local number = tonumber(self.editBox:GetText());

	if number then
		functions.hideUser();
		functions.disableLeader();
		
		RandomRoll(1,number);
	end

	self:Hide();
end


-----------------------
-----POPUP WINDOWS-----
-----------------------


--Pop up for manual rolls.
StaticPopupDialogs["LPTLootRoll_ManualRoll"] = 
{
	text = "Enter roll (Max: 9999):",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = 1,
	maxLetters = 4,
	OnShow = 
	function(self)
		local editBox = self.editBox;
		editBox:SetText("");
		editBox:SetFocus();
	end,
	OnAccept = 
	function(self)
		manualRoll(self);
	end,
	EditBoxOnEnterPressed = 
	function(self)
		manualRoll(self:GetParent());
	end,
	EditBoxOnEscapePressed = 
	function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting history.
StaticPopupDialogs["LPTLootRoll_ClearHistory"] = 
{
	text = "Are you sure you want to reset roll history?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self)
		functions.resetItemAndRollHistory();
		functions.printOut("LPT Loot Roll: Item history has been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting locations.
StaticPopupDialogs["LPTLootRoll_ResetPosition"] = 
{
	text = "Are you sure you want to reset window positions?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self)
		functions.resetUserWindow();
        functions.resetLeaderWindow();
        functions.resetItemListWindow();
		functions.printOut("LPT Loot Roll: Location of windows have been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for sending unwanted loot.
StaticPopupDialogs["LPTLootRoll_SendTradeable"] =
{
	text = "Do you want to list %s for trade? If accepted itemlink will be sent to the player below.",
	button1 = "Yes",
	button2 = "No",
	hasItemFrame = 1,
	hasEditBox = 1,
	maxLetters = 12,
	OnShow = 
	function(self)
		local editBox = self.editBox;

		if llrSettings.masterLooterMode then
			masterLooter = llrSettings.masterLooter;
		end

		if masterLooter == "" or not (UnitPlayerOrPetInRaid(masterLooter) and (UnitIsGroupAssistant(masterLooter) or UnitIsGroupLeader(masterLooter))) then
			masterLooter = functions.getRaidLeader() or "";
		end

		editBox:SetPoint("BOTTOM", 0, 90);
		editBox:SetText(masterLooter);
		editBox:SetAutoFocus(false);
		editBox:ClearFocus();
	end,
	OnAccept = 
	function(self)
		sendTradeableFunc(self);
	end,
	EditBoxOnEnterPressed = 
	function(self)
		sendTradeableFunc(self:GetParent());
	end,
	EditBoxOnEscapePressed = 
	function(self)
		self:ClearFocus();
	end,
	OnHide = function (self, data)
		functions.popTradeable();
	end,
	timeout = 0,
	whileDead = 1,
	noCancelOnReuse = 1,
	hideOnEscape = 1
};

--Pop up for accepting settings rolls.
StaticPopupDialogs["LPTLootRoll_SettingShare"] = {
	text = "Are you sure you want to change roll values?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self, msg)
		--If user accepts the data, parse it to numbers and store it in settings.
		local main, off, mog = strsplit("-", msg);

		main = tonumber(main);
		off  = tonumber(off);
		mog  = tonumber(mog);
		
		if main and off and mog then
			llrSettings.mainRoll = main;
			llrSettings.offRoll = off;
			llrSettings.mogRoll = mog;
		end
		
		functions.printOut("Roll values have been changed to:")
		functions.printOut("Main: " .. main);
		functions.printOut("Off: " .. off);
		functions.printOut("Mog: " .. mog);
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for distributing loot.
StaticPopupDialogs["LPTLootRoll_DistributeItem"] =
{
	text = "Give item to %s?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self, msg)
		local name, rollType = strsplit("-", msg);

		if not name or not rollType then
			return;
		end

		--If user accepts, tell who wins it in raidwarning.
		SendChatMessage("Loot goes to " .. name .. " for " .. rollType .. " roll." , "RAID_WARNING", nil, nil);
		variables.rollHistory[1].distributed = true;

		--If whisperlistener is off or if whisper listener is on with no more items to distribute then close the leader window.
		if not (llrSettings.whisperListener and #variables.itemsToRoll > 0) then
			functions.closeLeaderWindow();
		end
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};